@@ -27,36 +27,58 @@ def read_fastq(stream):
     """ Return an iterator over FASTQ entries in a stream
     (an iterable) """
 
-    header = None
     sequence = list()
     quality = list()
-    is_seq = False
-    for row in stream:
+    stream_iter = iter(stream)
+    row = stream_iter.next()
+    if row.startswith('@'):
+        header = row.rstrip()
+    else:
+        raise FastqError('First row does not look like a Fastq header.')
+    
+    for row in stream_iter:
         row = row.rstrip()
-        if row.startswith('@'):
-            # header
-            if header is not None:
-                sequence = ''.join(sequence)
-                quality = ''.join(quality)
-                yield Entry(header, sequence, quality)
-            #
+        if header is not None:
+            # sequence
+            sequence.append(row)
+            for row in stream_iter:
+                row = row.rstrip()
+                if row.startswith('+'):
+                    if len(sequence) == 0:
+                        raise FastqError('Empty Sequence')
+                    break
+                else:
+                    sequence.append(row)
+            # quality
+            for row in stream_iter:
+                row = row.rstrip()
+                if row.startswith('@'):
+                    if len(quality) == len(sequence):
+                        break
+                    else:
+                        quality.append(row)
+                elif row.startswith('+'):
+                    if len(quality) >= len(sequence):
+                        raise FastqError('Problem with the Fastq file '+\
+                                         '(nearby header is %s)' % header)
+                    else:
+                        quality.append(row)
+                else:
+                    quality.append(row)
+            sequence = ''.join(sequence)
+            quality = ''.join(quality)
+            yield Entry(header, sequence, quality)
+            # reset for next round
             header = row
             sequence = list()
             quality = list()
-            is_seq = True
-        elif row.startswith('+'):
-            # quality
-            is_seq = False
-        else:
-            # append sequence
-            if is_seq:
-                sequence.append(row)
-            else:
-                quality.append(row)
+    # do not miss the last entry
     sequence = ''.join(sequence)
+    quality = ''.join(quality)
     if header is None:
         raise FastqError('Missing FASTQ header.')
-    yield Entry(header, sequence, quality)        
+    if row != header:
+        yield Entry(header, sequence, quality)        
 
 from ngs_plumbing import report
 import csv
